/* Definitions for listing the contents of directories.
   Copyright (C) 1998 Rex Barzee

   This file is part of cls which lists contents of directories in color.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLS_INCLUDED
#define CLS_INCLUDED

#include "format.h"
#include <sys/types.h>

typedef enum { Stream, SingleColumn, ColumnDown, ColumnAcross, Long } PrintForm;
typedef enum { Modification, Status, Access } UseTime;
typedef enum { Ascii, Octal, Question } NonGraphic;

struct clsent {
  const char *name;
  const char *user;
  const char *group;
  const char *target_name;
  int len;
  clstype type;
  clstype target_type;
  uid_t   uid;
  gid_t   gid;
  ino_t   ino;
  mode_t  mode;
  nlink_t nlink;  /* number of links to the file */
  dev_t   rdev;   /* device ID (if file is character or block special) */
  off_t   size;   /* file size in bytes (if file is a regular file) */
  time_t  atime;  /* time of last access */
  time_t  mtime;  /* time of last data modification */
  time_t  ctime;  /* time of last status change */
  long    blksize;  /* a filesystem-specific preferred I/O block size
                       for this object.  In some filesystem types, this
                       may vary from file to file. */
  long    blocks;   /* number of blocks of a filesystem-specific size
                       allocated for this object */
};

extern termtype f_term;
extern bool f_listdot;      /* list files beginning with . */
extern bool f_listcurrent;  /* list current and parent directories, i.e. . and .. */
extern bool f_listdir;      /* list actual directory, not contents */
extern bool f_recursive;
extern NonGraphic f_nongraphic; /* print nongraphic characters as ascii,
                                   octal, or question marks */

extern PrintForm f_printform;
extern bool f_inode;    /* print inode number */
extern bool f_blocks;   /* print block size of file */
extern bool f_total;    /* print total number of blocks */
extern bool f_noowner;  /* long listing without owner */
extern bool f_nogroup;  /* long listing without group */
extern bool f_numbers;  /* long listing with user and group ids */
extern bool f_type;     /* print type character for non-regular files */
extern bool f_link;     /* List the file or directory to which the link
                           refers rather than the link itself. */

extern bool f_reversesort;  /* reverse whatever sort is used */
extern bool f_timesort;
extern UseTime f_usetime;

/* print.c */
typedef void (* PrintFcn)(const struct clsent *, int, int, int);
typedef int  (* NameFcn)(const char *);
extern PrintFcn PrintFunc;
extern NameFcn  NameLen;
extern void PrintInit();
extern void PrintDir(const char *name);

/* sort.c */
typedef int (* CmpFcn)(const void *, const void *);
extern bool   IsDirectory(const struct clsent *e);
extern CmpFcn SelectNonRecursSort();
extern CmpFcn SelectRecursSort();

/* time32.c */
extern struct tm *localtime32(time_t *clock);
extern struct tm *gmtime32(time_t *clock);
extern char *asctime32(struct tm *tm);
extern char *ctime32(time_t *clock);

/* user.c */
extern const char *GetUser(uid_t uid);
extern const char *GetGroup(gid_t gid);

#endif
