/* Functions and variables to print in color in different terminal types.
   Copyright (C) 1998 Rex Barzee

   This file is part of cls which lists contents of directories in color.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "format.h"
#include <stdlib.h>
#include <unistd.h>

const int TYPE_LENGTHS[] = { 0, 0, 0, 0,
                             1, 0, 1, 0, 1, 0, 1, 0, 1, 0,
                             0, 0,
                             1, 0, 1, 0, 1, 0, 1, 0, 1, 0 };

const char NONE[] = "";

const char DT_OFF[]      = "[0m",
           DT_YELLOW[]   = "[33m",
           DT_OFF_DIR[]  = "[0m/",
           DT_GREEN[]    = "[32m",
           DT_OFF_EXEC[] = "[0m*",
           DT_RED[]      = "[31m",
           DT_OFF_FIFO[] = "[0m|",
           DT_MAGENTA[]  = "[35m",
           DT_OFF_SOCK[] = "[0m=",
           DT_CYAN[]     = "[36m",
           DT_OFF_LINK[] = "[0m@",
           DT_BLUE[] = "[34m";

const char HP_OFF[]      = "&v0S",
           HP_YELLOW[]   = "&v3S",
           HP_OFF_DIR[]  = "&v0S/",
           HP_GREEN[]    = "&v2S",
           HP_OFF_EXEC[] = "&v0S*",
           HP_RED[]      = "&v1S",
           HP_OFF_FIFO[] = "&v0S|",
           HP_MAGENTA[]  = "&v5S",
           HP_OFF_SOCK[] = "&v0S=",
           HP_CYAN[]     = "&v6S",
           HP_OFF_LINK[] = "&v0S@",
           HP_BLUE[]  = "&v4S",
           HP_BLACK[] = "&v7S";

const char OPEN_DIR[]  = "/",
           OPEN_EXEC[] = "*",
           OPEN_FIFO[] = "|",
           OPEN_SOCK[] = "=",
           OPEN_LINK[] = "@";


const char *DT_FORMATS[] = {
  NONE,       NONE,   NONE,                 /* NoType */
  NONE,       NONE,   "white",              /* Ordinary */
  DT_YELLOW,  DT_OFF, "[33myellow[0m",  /* Directory */
  DT_GREEN,   DT_OFF, "[32mgreen[0m",   /* Executable */
  DT_RED,     DT_OFF, "[31mred[0m",     /* Fifo */
  DT_MAGENTA, DT_OFF, "[35mmagenta[0m", /* Socket */
  DT_CYAN,    DT_OFF, "[36mcyan[0m",    /* SymLink */
};

const char *HP_FORMATS[] = {
  NONE,       NONE,   NONE,                  /* NoType */
  NONE,       NONE,   "white",               /* Ordinary */
  HP_YELLOW,  HP_OFF, "&v3Syellow&v0S",  /* Directory */
  HP_GREEN,   HP_OFF, "&v2Sgreen&v0S",   /* Executable */
  HP_RED,     HP_OFF, "&v1Sred&v0S",     /* Fifo */
  HP_MAGENTA, HP_OFF, "&v5Smagenta&v0S", /* Socket */
  HP_CYAN,    HP_OFF, "&v6Scyan&v0S",    /* SymLink */
};

const char *OPEN_FORMATS[] = {
  NONE, NONE, NONE,
  NONE, NONE, "white",
  NONE, NONE, "yellow",
  NONE, NONE, "green",
  NONE, NONE, "red",
  NONE, NONE, "magenta",
  NONE, NONE, "cyan",
};

const char **FORMATS;


clstype FileType(mode_t m) {
  if (S_ISLNK(m))
    return SymLink;
  else if (S_ISDIR(m))
    return Directory;
  else if (S_ISFIFO(m))
    return Fifo;
  else if (S_ISSOCK(m))
    return Socket;
#ifdef _HPUX_SOURCE
  else if (S_ISNWK(m))
    return Socket;
#endif
  else if (m & (S_IXUSR | S_IXGRP | S_IXOTH))
    return Executable;
  else
    return Ordinary;
}


termtype Terminal(bool use_escape) {
  termtype term = Open;
  const char *p;

  if (use_escape || isatty(STDOUT_FILENO)) {
    if ((p = getenv("TERM")) != NULL) {
      if (strcmp(p, "ansi_xterm") == 0 ||
          strcmp(p, "color_xterm") == 0 ||
          strcmp(p, "dtterm") == 0 ||    /* Default terminal shipped with CDE */
          strcmp(p, "iris-ansi") == 0 || /* SGI default terminal */
          strcmp(p, "kterm") == 0 ||
          strcmp(p, "nxterm") == 0 ||
          strcmp(p, "rxvt") == 0 ||
          (strncmp(p, "vt2", 3) == 0 && strlen(p) == 5))
        term = Dtterm;
      else if (strcmp(p, "hp") == 0 || strcmp(p, "hpterm") == 0) 
        term = Hpterm;
    }
  }
  return term;
}
