/* Sort contents of directories.
   Copyright (C) 1998 Rex Barzee

   This file is part of cls which lists contents of directories in color.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "cls.h"
#include <string.h>

bool IsDirectory(const struct clsent *e) {
#if 0
  return e->type == Directory || e->target_type == Directory ? true : false;
#endif
  return e->type == Directory ? true : false;
}


static int NameCmp(const void *ent1, const void *ent2) {
  return strcmp(((struct clsent *)ent1)->name, ((struct clsent *)ent2)->name);
}


static int RevNameCmp(const void *ent1, const void *ent2) {
  return strcmp(((struct clsent *)ent2)->name, ((struct clsent *)ent1)->name);
}


static int ModNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e1->mtime > e2->mtime)
    return -1;
  else if (e1->mtime < e2->mtime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevModNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e2->mtime > e1->mtime)
    return -1;
  else if (e2->mtime < e1->mtime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int StatusNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e1->ctime > e2->ctime)
    return -1;
  else if (e1->ctime < e2->ctime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevStatusNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e2->ctime > e1->ctime)
    return -1;
  else if (e2->ctime < e1->ctime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int AccessNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e1->atime > e2->atime)
    return -1;
  else if (e1->atime < e2->atime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevAccessNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (e2->atime > e1->atime)
    return -1;
  else if (e2->atime < e1->atime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int TypeNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevTypeNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int TypeModNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e1->mtime > e2->mtime)
    return -1;
  else if (e1->mtime < e2->mtime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevTypeModNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e2->mtime > e1->mtime)
    return -1;
  else if (e2->mtime < e1->mtime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int TypeStatusNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e1->ctime > e2->ctime)
    return -1;
  else if (e1->ctime < e2->ctime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevTypeStatusNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e2->ctime > e1->ctime)
    return -1;
  else if (e2->ctime < e1->ctime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


static int TypeAccessNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e1->atime > e2->atime)
    return -1;
  else if (e1->atime < e2->atime)
    return 1;
  else
    return strcmp(e1->name, e2->name);
}


static int RevTypeAccessNameCmp(const void *ent1, const void *ent2) {
  const struct clsent *e1 = (struct clsent *)ent1;
  const struct clsent *e2 = (struct clsent *)ent2;

  if (!IsDirectory(e1) && IsDirectory(e2))
    return -1;
  else if (IsDirectory(e1) && !IsDirectory(e2))
    return 1;
  else if (e2->atime > e1->atime)
    return -1;
  else if (e2->atime < e1->atime)
    return 1;
  else
    return strcmp(e2->name, e1->name);
}


CmpFcn SelectNonRecursSort() {
  if (f_timesort) {
    switch (f_usetime) {
      case Modification:
        return f_reversesort ? RevModNameCmp : ModNameCmp;
      case Status:
        return f_reversesort ? RevStatusNameCmp : StatusNameCmp;
      case Access:
        return f_reversesort ? RevAccessNameCmp : AccessNameCmp;
    }
    return f_reversesort ? RevModNameCmp : ModNameCmp;
  }
  return f_reversesort ? RevNameCmp : NameCmp;
}


CmpFcn SelectRecursSort() {
  if (f_timesort) {
    switch (f_usetime) {
      case Modification:
        return f_reversesort ? RevTypeModNameCmp : TypeModNameCmp;
      case Status:
        return f_reversesort ? RevTypeStatusNameCmp : TypeStatusNameCmp;
      case Access:
        return f_reversesort ? RevTypeAccessNameCmp : TypeAccessNameCmp;
    }
    return f_reversesort ? RevTypeModNameCmp : TypeModNameCmp;
  }
  return f_reversesort ? RevTypeNameCmp : TypeNameCmp;
}
